/* --------------------------------------------------------------------------------------------------- 
 * BuildTool - Custom / Extra Javascript
 * 
 * @author	Robin Bonnes <http://robinbonnes.nl/>
 * @version	1.5
 *
 * Copyright (C) 2013 - 2014 Robin Bonnes. All rights reserved.
 * 
 *
 * This is the code that will be loaded after the app has been initialized. It will:
 *
 * - Set Push Notifications
 * - Set Stats
 * - Set Ads
 * - Set iOS specific options
 * - Set Android specific options
 *
 * NOTE: Don't change this file, as this will be re-included after every app change in BuildTool.
 * -------------------------------------------------------------------------------------------------- */

/*
 * BuildTool Settings
 */
var config = "bt_config.xml";
var pushNotification;
var gaPlugin;
var statusBar;

/*
 * BuildTool Extra Settings
 */
var bt_location;
var bt_app_id;
var push_android_enable;
var push_android_senderid;
var push_ios_enable;
var stats_ga_enable;
var stats_ga_accountid;
var ads_iads_enable;
var ads_iads_position;
var specific_ios_showstatusbar;
var specific_ios_overlaystatusbar;
var specific_ios_statusbarstyle;

/* --------------------------------------------------------------------------------------------------- */

/*
 * When Device Ready
 */
 
document.addEventListener("deviceReady", function()
{
	// Get all the extra options from the config.
	$.get(config, function(data)
	{
		if(data)
		{		
			// Get data
			bt_location						= $(data).find('bt_extra[setting=bt_location]').attr('value');
			bt_app_id						= $(data).find('bt_extra[setting=bt_app_id]').attr('value');
			push_android_enable				= $(data).find('bt_extra[setting=push_android_enable]').attr('value');
			push_android_senderid			= $(data).find('bt_extra[setting=push_android_senderid]').attr('value');
			push_ios_enable					= $(data).find('bt_extra[setting=push_ios_enable]').attr('value');
			stats_ga_enable					= $(data).find('bt_extra[setting=stats_ga_enable]').attr('value');
			stats_ga_accountid				= $(data).find('bt_extra[setting=stats_ga_accountid]').attr('value');
			ads_iads_enable					= $(data).find('bt_extra[setting=ads_iads_enable]').attr('value');
			ads_iads_position				= $(data).find('bt_extra[setting=ads_iads_position]').attr('value');
			specific_ios_showstatusbar 		= $(data).find('bt_extra[setting=specific_ios_showstatusbar]').attr('value');
			specific_ios_overlaystatusbar 	= $(data).find('bt_extra[setting=specific_ios_overlaystatusbar]').attr('value');
			specific_ios_statusbarstyle 	= $(data).find('bt_extra[setting=specific_ios_statusbarstyle]').attr('value');

			// Setup
			bt_setup();
		}
	});
}, false);

/* --------------------------------------------------------------------------------------------------- */

/*
 * Setup
 */
 
function bt_setup()
{
	// Enable Push
	if(push_android_enable == "1" || push_ios_enable == "1")
	{
		pushNotification = window.plugins.pushNotification;
	}
	
	// Setup Android Push Notifications
	if(push_android_enable == "1")
	{
		bt_setup_push_android();
	}
	
	// Setup iOS Push Notifications
	if(push_ios_enable == "1")
	{
		bt_setup_push_ios();
	}
	
	// Setup Google Analytics
	if(stats_ga_enable == "1")
	{
		bt_setup_stats();
	}
	
	// Setup Ads
	if(ads_iads_enable == "1")
	{
		bt_setup_ads();
	}
	
	// Hide Statusbar on iOS7
	if(specific_ios_showstatusbar != "1")
	{
		bt_setup_ios_hidestatusbar();
	}
	else
	{
		// Overlay Statusbar on iOS7
		if(specific_ios_overlaystatusbar == "1")
		{
			bt_setup_ios_overlaystatusbar();
		}
	}
}

/* --------------------------------------------------------------------------------------------------- */

/*
 * Setup Push Notifications on Android
 */
function bt_setup_push_android()
{
	try 
	{ 
		if (device.platform == 'android' || device.platform == 'Android')
		{
			pushNotification.register(
				successHandler,
				errorHandler,
				{
					"senderID": push_android_senderid,
					"ecb": "onNotificationGCM"
				}
			);
		}
	}
	catch(err) 
	{
		//console.log(err.message);
	} 
}

/*
 * Setup Push Notifications on iOS
 */
function bt_setup_push_ios()
{
	try 
	{ 
		if (device.platform == 'ios' || device.platform == 'iOS')
		{
			pushNotification.register(
				tokenHandler,
				errorHandler,
				{
					"badge":"true",
					"sound":"true",
					"alert":"true",
					"ecb":"onNotificationAPN"
				}
			);
		}
	}
	catch(err) 
	{
		//console.log(err.message);
	} 
}
 
/*
 * Setup Google Analytics
 */
function bt_setup_stats()
{
	// Try Local
	gaPlugin = window.plugins.gaPlugin;
    gaPlugin.init(successHandler, errorHandler, stats_ga_accountid, 10);
	gaPlugin.trackPage(successHandler, errorHandler, "/index");
}

/*
 * Setup Ads
 */
function bt_setup_ads()
{
	if (device.platform == 'ios' || device.platform == 'iOS')
	{
		window.addEventListener("orientationchange", window.plugins.iAd.orientationChanged, false);
		document.addEventListener("iAdBannerViewDidLoadAdEvent", iAdBannerViewDidLoadAdEventHandler, false);
		document.addEventListener("iAdBannerViewDidFailToReceiveAdWithErrorEvent", iAdBannerViewDidFailToReceiveAdWithErrorEventHandler, false);
		
		if(ads_iads_position == "top")
		{
			var adAtBottom = false;
		}
		else
		{
			var adAtBottom = true;
		}
		
		setTimeout(function()
		{
			window.plugins.iAd.prepare(adAtBottom);
		}, 1000);
		
		window.plugins.iAd.orientationChanged(true);
	}
}

/*
 * Hide Statusbar on iOS7
 */
function bt_setup_ios_hidestatusbar()
{
	if (device.platform == 'ios' || device.platform == 'iOS')
	{
		statusBar = window.plugins.StatusBar;
		statusBar.hide();
	}
}

/*
 * Overlay Statusbar on iOS7
 */
function bt_setup_ios_overlaystatusbar()
{
	if (device.platform == 'ios' || device.platform == 'iOS')
	{
		statusBar = window.plugins.StatusBar;
		statusBar.overlaysWebView(true);
		
		if(specific_ios_statusbarstyle == "default")
		{
			statusBar.styleDefault();
		}
		else if(specific_ios_statusbarstyle == "lightcontent")
		{
			statusBar.styleLightContent();
		}
		else if(specific_ios_statusbarstyle == "blackopaque")
		{
			statusBar.styleBlackOpaque();
		}
		else if(specific_ios_statusbarstyle == "blacktranslucent")
		{
			statusBar.styleBlackTranslucent();
		}
	}
}

/* --------------------------------------------------------------------------------------------------- */

function onNotificationAPN(e)
{
	if (e.alert)
	{
		 navigator.notification.alert(e.alert, apnHandler, "Notification", "OK");
	}
		
	if (e.sound)
	{
		var snd = new Media(e.sound);
		snd.play();
	}

	if (e.badge)
	{
		pushNotification.setApplicationIconBadgeNumber(successHandler, errorHandler, e.badge);
	}
}

function onNotificationGCM(e)
{
	switch( e.event )
	{
		case 'registered':
		if ( e.regid.length > 0 )
		{
			$.get( bt_location + "?push", {"app_id": bt_app_id, "device_id": e.regid, "os": "android"});
		}
		break;
		
		case 'message':
			navigator.notification.alert(e.payload.message, successHandler, "Notification", "OK");
		break;
		
		case 'error':
			// console.log(e.msg);
		break;
		
		default:
		break;
	}
}

function apnHandler(result)
{
	// Clear badge icon
	pushNotification.setApplicationIconBadgeNumber(successHandler, errorHandler, 0);
}

function tokenHandler(result)
{
	$.post( bt_location + "?push", {"app_id": bt_app_id, "device_id": e.regid, "os": "ios"});
}

function successHandler(result)
{
	// console.log(result);
}

function errorHandler(error)
{
	// console.log(error);
}

function onOrientationChange()
{
	// console.log(window.orientation);
}

function iAdBannerViewDidFailToReceiveAdWithErrorEventHandler(evt)
{
	// console.log(evt.error);
	window.plugins.iAd.showAd(false);
}

function iAdBannerViewDidLoadAdEventHandler(evt)
{
	window.plugins.iAd.showAd(true);
}