<?php

/* --------------------------------------------------------------------------------------------------- 
 * BuildTool - App Builder
 * 
 * @author	AppResponsive <http://robinbonnes.nl/>
 * @version	1.5
 *
 * Copyright (C) 2013 - 2014 AppResponsive. All rights reserved.
 * 
 * -------------------------------------------------------------------------------------------------- */

/* Check access */
if(!defined('access') or !access) die('This file cannot be directly accessed.');

class Bootstrap {

	/* Functions */
	public $f;
	
	/* Database */
	public $db;
	
	/* Singleton */
	public static $instance = null;
	public static function getInstance()
	{
		if ( self::$instance == null )
		{
			$className = __CLASS__;
			self::$instance = new $className();
		}
	
		return self::$instance;
	}

	/* Constructor */
	public function __construct()
	{
		self::$instance = $this;
		
		/* Get Definitions */
		$this->getDefinitions();
		
		/* Get Functions */
		$this->getFunctions();
		
		/* Set Defaults */
		$this->setDefaults();
		
		/* Start Template Engine */
		$this->startTemplateEngine();
		
		/* Check Install */
		$this->checkInstall();

		/* Start Database Engine */
		$this->startDatabaseEngine();
		
		/* Check API request/post */
		$this->checkAPI();
		
		/* Check login */
		$this->checkLogin();
		
		/* Load requested page */
		$this->loadPage();
	}
	
	/* Get Definitions */
	private function getDefinitions()
	{
		require_once( dirname(__FILE__) . '/definitions.php');
		new Definitions();
	}
	
	/* Get Functions */
	private function getFunctions()
	{
		require_once( dirname(__FILE__) . '/functions.php');
		$this->f = new Functions();
	}
	
	/* Set Defaults */
	private function setDefaults()
	{
		// Set Debug
		if( __DEBUG__ == false )
		{
			// Hide errors
			error_reporting(0);
			@ini_set('display_errors', false);
		}
		else
		{
			// Show Errors
			error_reporting(E_ALL);
			@ini_set('display_errors', true);
			//error_reporting(E_ALL & ~E_NOTICE);
		}
		
		// Set Session
		@session_save_path( __ROOT_DIR__ . "tmp/" );
		@session_set_cookie_params( 0 );
		@ini_set("session.gc_maxlifetime", "604800"); 
		@ini_set("session.cookie_lifetime", "0");
		session_start();
		
		// Set max filesize and postsize
		@ini_set('upload_max_filesize', '200M');
		ini_set('post_max_size', '200M');
		
		// Disable Execution Limit PHP (BuildTool uses sleep() to detect if download is available);
		@ini_set('max_execution_time', 0);
		@set_time_limit(0);
	}
	
	/* Start Template Engine */
	private function startTemplateEngine()
	{
		/* Set Encoding */
		@ini_set('default_charset', 'utf-8');
		
		/* Load Engine */
		include_once( __SYSTEM_DIR__ .'classes/template.class.php' );
	}

	/* Start Database Engine */
	private function startDatabaseEngine()
	{
		// Create database connection
		include_once( __SYSTEM_DIR__ .'classes/db.class.php' );
		require( __SYSTEM_DIR__ .'config.php');
		$this->db = new Database($config['db_host'], $config['db_user'], $config['db_pass'], $config['db_name']);
		
		if(count($this->db->errors) > 0)
		{
			// Show error
			$header = new Template("header.tpl.php");
			$header->set("title", "BuildTool - Error");
			$header->set("subtitle", "Oops! An error occured.");
			$footer = new Template("footer.tpl.php");
			$body = new Template("error.tpl.php");
			$body->set("header", $header->parse());
			$body->set("footer", $footer->parse());
			$body->set("errors", $this->db->errors);
			$body->render();
			die();
		}
	}
	
	/* Check Install */
	private function checkInstall()
	{
		if(!file_exists( __SYSTEM_DIR__ .'config.php'))
		{
			// Load installer
			require_once(__CONTROLLERS_DIR__ .'/installer.php');
			new Installer();
			die();
		}
		else
		{
			// Include Config
			require_once( __SYSTEM_DIR__ .'config.php');
			
			if($config['bt_version'] != __BT_VERSION__)
			{
				// Load updater
				require_once(__CONTROLLERS_DIR__ .'/installer.php');
				new Installer();
				die();
			}
		}
	}
	
	/* Check if an API request is done */
	public function checkAPI()
	{
		// Push Notifications
		if(isset($_GET['push']))
		{
			// Get registration info
			$app_id = $_GET['app_id'];
			$device_id = $_GET['device_id'];
			$os = $_GET['os'];
			
			// Check if device_id is already in database
			$this->db->prepare("SELECT * FROM push_devices WHERE app_id = ? AND device_id = ?");
			$this->db->bindParam(1, $app_id);
			$this->db->bindParam(2, $device_id);
			$this->db->execute();
			$device = $this->db->fetch();

			// If not, add to database
			if(!$device)
			{
				$this->db->prepare("INSERT INTO push_devices(app_id, device_id, os) VALUES (?, ?, ?)");
				$this->db->bindParam(1, $app_id);
				$this->db->bindParam(2, $device_id);
				$this->db->bindParam(3, $os);
				$this->db->execute();
			}
			
			die(" en dit");
		}
	}
	
	/* Check Login */
	public function checkLogin()
	{
		if(isset($_SESSION["user"]) && isset($_SESSION["hash"]))
		{
			$this->db->prepare("SELECT * FROM users WHERE username = ?");
			$this->db->bindParam(1, $_SESSION["user"]);
			$this->db->execute();
			$user = $this->db->fetch();

			if( $user )
			{
				if(!(md5($user["username"] . $user["password"] . $_SERVER['REMOTE_ADDR']) == $_SESSION["hash"]))
				{			
					// Destroy Session
					session_destroy();
					
					// Load login
					require_once(__CONTROLLERS_DIR__ .'/login.php');
					new Login();
					die();
				}
			}
		}
		else
		{
			// Load login
			require_once(__CONTROLLERS_DIR__ .'/login.php');
			new Login();
			die();
		}
	}
		
	/* Load Requested Page */
	public function loadPage()
	{
		// Load page
		if(isset($_GET['page']))
		{
			$classname = ucfirst(strtolower($_GET['page']));
			if(file_exists(__CONTROLLERS_DIR__ .'/' . $_GET['page'] . '.php'))
			{
				require_once(__CONTROLLERS_DIR__ .'/' . $_GET['page'] . '.php');
				if(get_class($this) != $classname)
				{
					if(isset($_GET[1]))
					{
						new $classname($_GET[1]);
					}
					else
					{
						new $classname();
					}
					die();
				}
			}
			else
			{
				// Load dashboard (default/landing)
				require_once(__CONTROLLERS_DIR__ .'/dashboard.php');
				new Dashboard();
				die();
			}
		}
		else
		{
			// Load dashboard (default/landing)
			require_once(__CONTROLLERS_DIR__ .'/dashboard.php');
			new Dashboard();
			die();
		}
	}
}
?>