<?php

/* --------------------------------------------------------------------------------------------------- 
 * BuildTool - App Builder
 * 
 * @author	AppResponsive <http://robinbonnes.nl/>
 * @version	1.5
 *
 * Copyright (C) 2013 - 2014 AppResponsive. All rights reserved.
 * 
 * -------------------------------------------------------------------------------------------------- */

/* Check access */
if(!defined('access') or !access) die('This file cannot be directly accessed.');

class Database
{
	public $errors		=	array();
	private $dbconn;
	private $sth;

	/* Singleton */
	public static $instance = null;
	public static function getInstance()
	{
		if ( self::$instance == null )
		{
			$className = __CLASS__;
			self::$instance = new $className();
		}
	
		return self::$instance;
	}
	
	public function __construct($host, $dbuser, $dbpwd, $dbname)
	{
		self::$instance = $this;

		// Settings
		$configuration = array(
			"hostname" => $host,
			"username" => $dbuser,
			"password" => $dbpwd,
			"database" => $dbname,
		);

		try
		{
			$this->configuration = $configuration;
			$this->dbconn = new PDO($this->createdsn($configuration), $configuration['username'], $configuration['password']);
			return $this->dbconn;
		}
		catch (Exception $e)
		{
			$this->errors['Database'] = $e->getMessage();
		}
	}
	
	private function createdsn($settings)
	{
		return 'mysql:host='.$settings['hostname'].';dbname='.$settings['database'];
	}

	public function query($query)
	{
		return $this->dbconn->query($query);
	}

	public function prepare($query)
	{
		$this->sth = $this->dbconn->prepare($query);
	}
	
	public function bindParam($param, $value)
	{
		$this->sth->bindParam($param, $value);
	}
	
	public function execute()
	{
		$this->sth->execute();
	}
	
	public function fetch()
	{
		return $this->sth->fetch();
	}
	
	public function fetchAll()
	{
		return $this->sth->fetchAll();
	}
	
	public function __destruct()
	{
		$this->dbconn = null;
		$this->sth = null;
	}
}
?>