<?php

/* --------------------------------------------------------------------------------------------------- 
 * BuildTool - App Builder
 * 
 * @author	AppResponsive <http://robinbonnes.nl/>
 * @version	1.5
 *
 * Copyright (C) 2013 - 2014 AppResponsive. All rights reserved.
 * 
 * -------------------------------------------------------------------------------------------------- */

/* Check access */
if(!defined('access') or !access) die('This file cannot be directly accessed.');

class Imageeditor
{
	private $image;
	private $width;
	private $height;
	private $resizedimage;

	function __construct($file)
	{
        if (!(is_readable($file) && is_file($file))) {
            throw new InvalidArgumentException("Image file $file is not readable");
        }

        if (is_resource($this->image)) {
            imagedestroy($this->image);
        }

        list ($this->width, $this->height, $type) = getimagesize($file);

        switch ($type) {
            case IMAGETYPE_GIF  :
                $this->image = imagecreatefromgif($file);
                break;
            case IMAGETYPE_JPEG :
                $this->image = imagecreatefromjpeg($file);
                break;
            case IMAGETYPE_PNG  :
                $this->image = imagecreatefrompng($file);
                break;
            default             :
                $this->image = imagecreatefrompng($file);
        }

	}

	public function resizeImage($width, $height)
	{
		// Transparancy
		$this->resizedimage = imagecreatetruecolor($width, $height);
		imagesavealpha($this->resizedimage,true);
		$transparent = imagecolorallocatealpha($this->resizedimage, 255, 255, 255, 127);
		imagefill($this->resizedimage, 0, 0, $transparent);
		
		// Resize
		imagecopyresampled($this->resizedimage, $this->image, 0, 0, 0, 0, $width, $height, $this->width, $this->height);
		
		return $this;
	}

	public function saveImage($path)
	{
		// PNG Transparency
		imagealphablending($this->resizedimage, false);
		imagesavealpha($this->resizedimage,true);
		
		// Save Image
		if (!imagepng($this->resizedimage, $path)) {
			throw new RuntimeException;
		}

		// Clear memory
		imagedestroy($this->resizedimage);
		imagedestroy($this->image);
	}

}
?>
