<?php

/* --------------------------------------------------------------------------------------------------- 
 * BuildTool - App Builder
 * 
 * @author	AppResponsive <http://robinbonnes.nl/>
 * @version	1.5
 *
 * Copyright (C) 2013 - 2014 AppResponsive. All rights reserved.
 * 
 * -------------------------------------------------------------------------------------------------- */

/* Check access */
if(!defined('access') or !access) die('This file cannot be directly accessed.');

class Dashboard {

	/* Database connection */
	private $db;
	
	/* Stats array */
	public $stats;
	
	/* New Templates array */
	public $templates;
	
	/* Blog Posts array */
	public $blogfeed;
	
	/* Constructor */
	public function __construct()
	{
		/* Assign Database */
		$this->db = Database::getInstance();
		
		/* Check if POST/GET is done. */
		$this->check();
		
		/* Check for updates */
		$this->checkUpdates();
		
		/* Get the latest templates */
		$this->getLatestTemplates();
		
		/* Get the statistics */
		$this->getStats();
		
		/* Show Dashboard. */
		$this->showDashboard();
	}
	
	/* Check if some request is done. */
	private function check()
	{
		// Logout
		if(isset($_GET['logout']))
		{
			// Destroy Session
			session_destroy();
			$host	=	"http://" . $_SERVER["HTTP_HOST"] . rtrim(dirname($_SERVER['PHP_SELF']), '/\\');
			header("Location: $host");
		}
		
		// Action
		if(isset($_GET['action']))
		{
			$header = new Template("header.tpl.php");
			$header->set("title", "BuildTool - Dashboard");
			$header->set("subtitle", "Overview of BuildTool.");
			$footer = new Template("footer.tpl.php");
			$body = new Template("dashboard.tpl.php");
			$body->set("header", $header->parse());
			$body->set("footer", $footer->parse());
			$body->set("username", $_SESSION["user"]);
		
			// Update template
			if($_GET['action'] == "update")
			{
				if(isset($_GET['package']) && $_GET['package'] != "")
				{
					$ch = curl_init();
					$source = __BT_TEMPLATE_DOWNLOAD__ . $_GET['package'];
					curl_setopt($ch, CURLOPT_URL, $source);
					curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
					$data = curl_exec ($ch);
					curl_close ($ch);
					$destination = __UPLOAD_DIR__ . "templates/" . $_GET['package'];
					$file = fopen($destination, "w+");
					fputs($file, $data);
					fclose($file);
					$body->set("success", "Template succesfully updated!<br>Make sure to rebuild your apps using this template for the newest version.");
				}
				else
				{
					// ERROR: No correct package
					$body->set("error", "ERROR: No correct package specified!");
				}
			}
			
			// Download template
			elseif($_GET['action'] == "download")
			{		
				if(isset($_GET['package']) && !empty($_GET['package']))
				{
					// Get package info
					$this->getLatestTemplates();
					foreach($this->templates as $template)
					{
						if($template['package'] == $_GET['package'])
						{
							$template_name = $template['title'];
							$template_description = $template['description'];
						}
					}
					
					// Download
					$ch = curl_init();
					$source = __BT_TEMPLATE_DOWNLOAD__ . $_GET['package'];
					curl_setopt($ch, CURLOPT_URL, $source);
					curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
					$data = curl_exec ($ch);
					curl_close ($ch);
					$destination = __UPLOAD_DIR__ . "templates/" . $_GET['package'];
					$file = fopen($destination, "w+");
					fputs($file, $data);
					fclose($file);
					
					// Install
					$this->db->prepare("INSERT INTO templates(template_name, template_description, template_package) VALUES (?, ?, ?)");
					$this->db->bindParam(1, $template_name);
					$this->db->bindParam(2, $template_description);
					$this->db->bindParam(3, $_GET['package']);
					$this->db->execute();
					
					$body->set("success", "Template succesfully downloaded and installed!");
				}
				else
				{
					// ERROR: No correct package
					$body->set("error", "ERROR: No correct package specified!");
				}
			}
			else
			{
				// ERROR: No correct action
				$body->set("error", "ERROR: No correct action specified!");
			}
			
			$this->getLatestTemplates();
			$this->getStats();
			$body->set("stats", $this->stats);
			$body->set("templates", $this->templates);
			$body->render();
			die();
		}
		
		// Load page
		if(isset($_GET['page']))
		{
			$classname = ucfirst(strtolower($_GET['page']));
			if(file_exists(__CONTROLLERS_DIR__ .'/' . $_GET['page'] . '.php'))
			{
				require_once(__CONTROLLERS_DIR__ .'/' . $_GET['page'] . '.php');
				if(get_class($this) != $classname)
				{
					if(isset($_GET[1]))
					{
						new $classname($_GET[1]);
					}
					else
					{
						new $classname();
					}
					die();
				}
			}
		}
		
	}
	
	/* Check Updates */
	public function checkUpdates()
	{
		if(isset($_GET["getupdatestatus"]) || isset($_GET["getupdateinfo"]) || isset($_GET["getupdate"]))
		{
			// Get version
			$version = __BT_VERSION__;
			
			// Get purchaseid
			$this->db->prepare("SELECT value FROM settings WHERE setting = 'buildtool_purchaseid'");
			$this->db->execute();
			$purchaseid = $this->db->fetch();
			$purchaseid = $purchaseid[0];
			
			if(isset($_GET["getupdatestatus"]))
			{
				// Get update status
				$ch = curl_init();
				$source = __BT_UPDATE_CHECK__ . "?purchaseid=" . $purchaseid . "&version=" . $version;
				curl_setopt($ch, CURLOPT_URL, $source);
				curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
				$data = curl_exec($ch);
				curl_close($ch);
				
				if(!__HIDE_UPDATES__)
				{
					echo $data;
				}
			}

			if(isset($_GET["getupdateinfo"]))
			{
				// Get update info
				$ch = curl_init();
				$source = __BT_UPDATE_INFO__ . "?purchaseid=" . $purchaseid . "&version=" . $version;
				curl_setopt($ch, CURLOPT_URL, $source);
				curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
				$data = curl_exec($ch);
				curl_close($ch);
				echo $data;
			}
			
			if(isset($_GET["getupdate"]))
			{
				$error = false;
				$header = new Template("header.tpl.php");
				$header->set("title", "BuildTool - Dashboard");
				$header->set("subtitle", "Overview of BuildTool.");
				$footer = new Template("footer.tpl.php");
				$body = new Template("dashboard.tpl.php");
				$body->set("header", $header->parse());
				$body->set("footer", $footer->parse());
				$body->set("username", $_SESSION["user"]);
				
				// Get update files as zip
				$ch = curl_init();
				$fp = fopen(__TEMP_DIR__ . 'update.zip', 'w');
				$source = __BT_UPDATE_DOWNLOAD__ . "?purchaseid=" . $purchaseid . "&version=" . $version;
				curl_setopt($ch, CURLOPT_URL, $source);
				curl_setopt($ch, CURLOPT_FILE, $fp);
				curl_exec($ch);
				curl_close($ch);
				fclose($fp);
				
				// Unpack and overwrite with current files
				$zip = new ZipArchive;
				if($zip->open(__TEMP_DIR__ . 'update.zip') === TRUE)
				{
					$zip->extractTo(__ROOT_DIR__);
					$zip->close();
				}
				else
				{
					$error = true;
				}

				if(!$error)
				{
					// Get update SQL settings
					//$data = @file_get_contents(__ROOT_DIR__ . 'update.sql');
					
					// Run SQL settings
					//$this->db->query($data);
					
					$body->set("success", "BuildTool is succesfully updated!");
				}
				else
				{
					$body->set("error", "Error occured! Please try again or read the documentation to update manually. (" . $error . ")");
				}
				
				// Remove update files
				@unlink(__TEMP_DIR__ . 'update.zip');
				@unlink(__ROOT_DIR__ . 'update.sql');
				
				$this->getLatestTemplates();
				$this->getStats();
				$body->set("stats", $this->stats);
				$body->set("templates", $this->templates);
				$body->render();
			}
			
			die();
		}
	}
	
	/* Get the latest templates */
	private function getLatestTemplates()
	{
		$this->templates = array();
		
		// Get server templates
		$ch = curl_init();
		$source = __BT_TEMPLATE_FEED__;
		curl_setopt($ch, CURLOPT_URL, $source);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		$data = curl_exec($ch);
		$data = json_decode($data);
		curl_close ($ch);
		
		// Get local templates
		$this->db->prepare("SELECT * FROM templates");
		$this->db->execute();
		$templates = $this->db->fetchAll();
			
		if($data)
		{
			foreach($data as $value)
			{
				$template['package'] = $value->template_package;
				$template['date'] = date("jS F, Y", strtotime($value->template_date));
				$template['title'] = $value->template_title;
				$template['description'] = $value->template_description;
				$template['action'] = "";
				
				foreach ($templates as $temp)
				{
					// Template exists
					if($temp['template_package'] == $value->template_package)
					{
						// Get version
						$version = "1.0";
						$zip = new ZipArchive;
						$res = $zip->open(__UPLOAD_DIR__ . "templates/" . $temp['template_package']);
						
						// Check content
						if ($res === TRUE)
						{
							$xmlDoc = new DOMDocument(); 
							if(is_numeric($zip->locateName("www/config.xml")))
							{
								$xmlDoc->loadXML($zip->getFromName('www/config.xml'));
							}
							elseif(is_numeric($zip->locateName("config.xml")))
							{
								$xmlDoc->loadXML($zip->getFromName('config.xml'));
							}
							$searchNode = $xmlDoc->getElementsByTagName( "widget" ); 
							foreach( $searchNode as $searchNode ) 
							{ 
								$version = $searchNode->getAttribute('version');
							}
							$zip->close();
						}
						
						if(version_compare($value->template_version, $version) > 0)
						{
							// Update
							$template['action'] = '<a href="?action=update&package=' . $value->template_package . '" class="button button-rounded button-highlight"><i class="icon-refresh"></i> UPDATE</a>';
						}
						else
						{
							// Installed
							$template['action'] = '<a href="#" class="button button-rounded button-action"><i class="icon-check-sign"></i> INSTALLED</a>';
						}
					}
				}

				if($template['action'] == "")
				{
					if($value->template_free == 1)
					{
						// Install
						$template['action'] = '<a href="?action=download&package=' . $value->template_package . '" class="button button-rounded button-primary"><i class="icon-cloud-download"></i> DOWNLOAD</a>';
					}
					else
					{
						// Buy Link
						$template['action'] = '<a href="' . $value->template_download . '" class="button button-rounded button-primary"><i class="icon-shopping-cart"></i> BUY</a>';
					}
				}
				
				$this->templates[] = $template;
			}
		}
	}
	
	/* Get the statistics */
	private function getStats()
	{
		$this->db->prepare("SELECT * FROM users");
		$this->db->execute();
		$this->stats['users'] = count($this->db->fetchAll());
		
		$this->db->prepare("SELECT * FROM templates");
		$this->db->execute();
		$this->stats['templates'] = count($this->db->fetchAll());
		
		$this->db->prepare("SELECT * FROM apps");
		$this->db->execute();
		$this->stats['apps'] = count($this->db->fetchAll());
	}
	
	/* Show Dashboard. */
	private function showDashboard()
	{
		$header = new Template("header.tpl.php");
		$header->set("title", "BuildTool - Dashboard");
		$header->set("subtitle", "Overview of BuildTool.");
		$footer = new Template("footer.tpl.php");
		$body = new Template("dashboard.tpl.php");
		$body->set("header", $header->parse());
		$body->set("footer", $footer->parse());
		$body->set("username", $_SESSION["user"]);
		$body->set("stats", $this->stats);
		$body->set("templates", $this->templates);
		$body->render();
	}
	
}
?>