<?php

/* --------------------------------------------------------------------------------------------------- 
 * BuildTool - App Builder
 * 
 * @author	AppResponsive <http://robinbonnes.nl/>
 * @version	1.5
 *
 * Copyright (C) 2013 - 2014 AppResponsive. All rights reserved.
 * 
 * -------------------------------------------------------------------------------------------------- */

/* Check access */
if(!defined('access') or !access) die('This file cannot be directly accessed.');

class Installer {

	/* arrray containing all requirements and results */
	private $requirements = array();
	
	/* array containing errors during install */
	private $errors = array();
	
	/* Variable whether or not BuildTool is updating */
	private $update = false;
	
	/* Default database settings */
	private $db_host = "localhost";
	private $db_database = "buildtool";
	private $db_username = "root";
	private $db_password = "";

	/* Constructor */
	public function __construct()
	{
		/* Check if Buildtool has been installed. */
		$this->checkInstalled();
		
		/* Check if BuildTool is updating */
		$this->checkUpdate();
		
		/* Check if Form has been posted. */
		$this->checkPost();
		
		/* Install */
		$this->install();
	}
	
	/* Check if Buildtool has been installed. */
	private function checkInstalled()
	{
		if(file_exists( __SYSTEM_DIR__ .'config.php'))
		{
			// Get Config
			require( __SYSTEM_DIR__ .'config.php');
			if($config['bt_version'] != __BT_VERSION__)
			{
				$this->update = true;
			}
			else
			{
				$header = new Template("header.tpl.php");
				$header->set("title", "Install BuildTool - Installed");
				$header->set("subtitle", "Follow the steps to install BuildTool");
				$footer = new Template("footer.tpl.php");
				$body = new Template("installer.tpl.php");
				$body->set("header", $header->parse());
				$body->set("footer", $footer->parse());
				$body->set("step", "5");
				$body->set("message_title", "BuildTool is already installed");
				$body->set("message", "You can reinstall BuildTool by deleting or renaming your '/system/config.php' file.");
				$body->render();
				die();
			}
		}
	}
	
	/* Check if BuildTool has to update */
	private function checkUpdate()
	{
		if($this->update)
		{
			// Allow registration setting
			$db->query("INSERT INTO `settings` (
				`setting` ,
				`value`
				)
				VALUES
				('allow_registration', '0');
			");
			
			// Add admin/client type for users
			$db->query("ALTER TABLE `users` ADD `type` VARCHAR( 255 ) NOT NULL");
			
			// Set admin as default on existing users
			$db->query("UPDATE `users` SET `type` = 'admin'");
			
			// Add client id for each app
			$db->query("ALTER TABLE `apps` ADD `clientid` INT( 11 ) unsigned NOT NULL");
			
			// Set first user as default client id for existing apps
			$db->query("UPDATE `apps` SET `clientid` = (SELECT `id` FROM `users` ORDER BY `id` ASC LIMIT 1)");
			
			// Set Push and Stats enabled options
			$db->query("ALTER TABLE `apps` ADD `push_android_enabled` TINYINT( 1 ) NOT NULL");
			$db->query("ALTER TABLE `apps` ADD `push_android_apikey` VARCHAR( 255 ) NOT NULL");
			$db->query("ALTER TABLE `apps` ADD `push_ios_enabled` TINYINT( 1 ) NOT NULL");
			$db->query("ALTER TABLE `apps` ADD `push_ios_keypass` VARCHAR( 255 ) NOT NULL");
			$db->query("ALTER TABLE `apps` ADD `push_ios_devmode` TINYINT( 1 ) NOT NULL");
			$db->query("ALTER TABLE `apps` ADD `stats_enabled` TINYINT( 1 ) NOT NULL");
			$db->query("ALTER TABLE `apps` ADD `stats_accountid` VARCHAR( 255 ) NOT NULL");
			
			// Create Push Devices table
			$db->query("CREATE TABLE `push_devices` (
			  `id` INT( 11 ) unsigned NOT NULL auto_increment,
			  `app_id` INT( 11 ) unsigned NOT NULL,
			  `device_id` mediumtext NOT NULL,
			  `os` varchar(255) NOT NULL,
			  PRIMARY KEY (`id`)
			)");
			
			/* Update Config */
			require( __SYSTEM_DIR__ .'config.php');
			$this->db_host = $config['db_host'];
			$this->db_database = $config['db_name'];
			$this->db_username = $config['db_user'];
			$this->db_password = $config['db_pass'];
			$this->writeConfig();
			
			/* Output */
			$header = new Template("header.tpl.php");
			$header->set("title", "BuildTool Updated");
			$header->set("subtitle", "BuildTool has been successfully updated!");
			$footer = new Template("footer.tpl.php");
			$body = new Template("installer.tpl.php");
			$body->set("header", $header->parse());
			$body->set("footer", $footer->parse());
			$body->set("step", "4");
			$body->set("update", true);
			$body->render();

			die();
		}
	}
	
	/* Check if Form has been posted. */
	private function checkPost()
	{
		if($_SERVER['REQUEST_METHOD'] == "POST")
		{
			if(isset($_POST['step1']))
			{
				$header = new Template("header.tpl.php");
				$header->set("title", "Install BuildTool - Step 2");
				$header->set("subtitle", "Follow the steps to install BuildTool");
				$footer = new Template("footer.tpl.php");
				$body = new Template("installer.tpl.php");
				$body->set("header", $header->parse());
				$body->set("footer", $footer->parse());
				$body->set("step", "2");
				
				/* Check if requirements are met. */
				if($this->checkRequirements())
				{
					/* Requirements are met, allow to move on to the next step. */
					$body->set("passed", true);
				}
				else
				{
					/* Requirements are not met, enable recheck button. */
					$body->set("passed", false);
				}
				
				$body->set("requirements", $this->requirements);
				$body->render();
			}
			elseif(isset($_POST['step2']))
			{				
				$header = new Template("header.tpl.php");
				$header->set("title", "Install BuildTool - Step 3");
				$header->set("subtitle", "Follow the steps to install BuildTool");
				$footer = new Template("footer.tpl.php");
				$body = new Template("installer.tpl.php");
				$body->set("header", $header->parse());
				$body->set("footer", $footer->parse());
				$body->set("step", "3");
				$body->render();
			}
			elseif(isset($_POST['step3']))
			{
				if(isset($_POST['db_host']) && isset($_POST['db_database']) && isset($_POST['db_username']) && isset($_POST['db_password'])
				&& isset($_POST['account_username']) && isset($_POST['account_password']) && isset($_POST['account_email']))
				{
					/* Install database */
					$this->installDatabase();
					
					/* Write Config */
					if(count($this->errors) == 0)
					{
						$this->db_host = $_POST['db_host'];
						$this->db_database = $_POST['db_database'];
						$this->db_username = $_POST['db_username'];
						$this->db_password = $_POST['db_password'];
						$this->writeConfig();
					}
					
					/* Output */
					if(count($this->errors) == 0)
					{
						/* Success, hooray! */
						$header = new Template("header.tpl.php");
						$header->set("title", "Install BuildTool - Finish");
						$header->set("subtitle", "Follow the steps to install BuildTool");
						$footer = new Template("footer.tpl.php");
						$body = new Template("installer.tpl.php");
						$body->set("header", $header->parse());
						$body->set("footer", $footer->parse());
						$body->set("login", $_POST['account_username'] . "/" . $_POST['account_password']);
						$body->set("step", "4");
						$body->render();
					}
					else
					{
						/* Display errors */
						$header = new Template("header.tpl.php");
						$header->set("title", "Install BuildTool - Step 3");
						$header->set("subtitle", "Follow the steps to install BuildTool");
						$footer = new Template("footer.tpl.php");
						$body = new Template("installer.tpl.php");
						$body->set("header", $header->parse());
						$body->set("footer", $footer->parse());
						$body->set("errorarray", $this->errors);
						$body->set("step", "3");
						$body->render();
					}
				}
				else
				{
					/* Not all fields are filled in */
					$header = new Template("header.tpl.php");
					$header->set("title", "Install BuildTool - Step 3");
					$header->set("subtitle", "Follow the steps to install BuildTool");
					$footer = new Template("footer.tpl.php");
					$body = new Template("installer.tpl.php");
					$body->set("header", $header->parse());
					$body->set("footer", $footer->parse());
					$body->set("step", "3");
					$body->set("requirements", $this->requirements);
					$body->render();
				}

			}
			else
			{
				/* Wrong post */
				$header = new Template("header.tpl.php");
				$header->set("title", "Install BuildTool");
				$header->set("subtitle", "Follow the steps to install BuildTool");
				$footer = new Template("footer.tpl.php");
				$body = new Template("installer.tpl.php");
				$body->set("header", $header->parse());
				$body->set("footer", $footer->parse());
				$body->set("step", "5");
				$body->set("message_title", "Oops, Something went wrong!");
				$body->set("message", "Please try again.");
				$body->render();
				
			}
			
			/* Very important */
			die();
		}
	}

	/* Install Buildtool. */
	private function install()
	{
		$header = new Template("header.tpl.php");
		$header->set("title", "Install BuildTool - Step 1");
		$header->set("subtitle", "Follow the steps to install BuildTool");
		$footer = new Template("footer.tpl.php");
		$body = new Template("installer.tpl.php");
		$body->set("header", $header->parse());
		$body->set("footer", $footer->parse());
		$body->set("step", "1");
		$body->render();
	}
	
	/* Checks if host meets the requirements. */
	private function checkRequirements()
	{
		/* Load Detect Class */
		include_once( __SYSTEM_DIR__ .'classes/detect.class.php' );
		$detect = new Detect();
		
		/* Check PHP ( > 5.1.9 ) */
		if($detect->hasPHP())
		{
			$this->requirements['PHP Version'] = "<img class=\"requirement-icon\" src=\"./assets/images/icons/tick.png\" width=\"32\" height=\"32\"><div class=\"requirement\">" . $detect->phpVersion . "</div>";
		}
		else
		{
			$this->requirements['PHP Version'] = "<img class=\"requirement-icon\" src=\"./assets/images/icons/cross.png\" width=\"32\" height=\"32\"><div class=\"requirement\"><font color=\"#FF0000\">UNAVAILABLE</font></div>";
		}
		
		/* Check MYSQL */
		if($detect->hasMySQL())
		{
			$this->requirements['MySQL'] = "<img class=\"requirement-icon\" src=\"./assets/images/icons/tick.png\" width=\"32\" height=\"32\"><div class=\"requirement\">AVAILABLE</div>";
		}
		else
		{
			$this->requirements['MySQL'] = "<img class=\"requirement-icon\" src=\"./assets/images/icons/cross.png\" width=\"32\" height=\"32\"><div class=\"requirement\"><font color=\"#FF0000\">UNAVAILABLE</font></div>";
		}

		/* Check GD Library */
		if($detect->hasGD())
		{
			$this->requirements['GD Library'] = "<img class=\"requirement-icon\" src=\"./assets/images/icons/tick.png\" width=\"32\" height=\"32\"><div class=\"requirement\">AVAILABLE</div>";
		}
		else
		{
			$this->requirements['GD Library'] = "<img class=\"requirement-icon\" src=\"./assets/images/icons/cross.png\" width=\"32\" height=\"32\"><div class=\"requirement\"><font color=\"#FF0000\">UNAVAILABLE</font></div>";
		}
		
		/* Check CURL */
		if($detect->hasCURL())
		{
			$this->requirements['CURL Library'] = "<img class=\"requirement-icon\" src=\"./assets/images/icons/tick.png\" width=\"32\" height=\"32\"><div class=\"requirement\">AVAILABLE</div>";
		}
		else
		{
			$this->requirements['CURL Library'] = "<img class=\"requirement-icon\" src=\"./assets/images/icons/cross.png\" width=\"32\" height=\"32\"><div class=\"requirement\"><font color=\"#FF0000\">UNAVAILABLE/font></div>";
		}
		
		/* Check Writable */
		if($detect->checkWritable(__UPLOAD_DIR__))
		{
			$this->requirements['Writable: "/uploads"'] = "<img class=\"requirement-icon\" src=\"./assets/images/icons/tick.png\" width=\"32\" height=\"32\"><div class=\"requirement\">YES</div>";
		}
		else
		{
			$this->requirements['Writable: "/uploads"'] = "<img class=\"requirement-icon\" src=\"./assets/images/icons/cross.png\" width=\"32\" height=\"32\"><div class=\"requirement\"><font color=\"#FF0000\">NO</font></div>";
		}
		
		if($detect->countErrors > 0) {
			return false;
		}
		
		return true;
	}
	
	/* Install the database. */
	private function installDatabase()
	{
		/* Load Database Class */
		include_once( __SYSTEM_DIR__ .'classes/db.class.php' );
		$db = new Database($_POST['db_host'], $_POST['db_username'], $_POST['db_password'], $_POST['db_database']);
		
		if(count($db->errors) == 0)
		{
			/* Connected, now install tables */
			
			// Remove old tables if exists
			$db->query("DROP TABLE users, templates, apps, settings");
			
			// Create users table
			$db->query("
			CREATE TABLE `users` (
			  `id` INT(11) unsigned NOT NULL auto_increment,
			  `type` VARCHAR( 255 ) NOT NULL,
			  `username` VARCHAR( 255 ) NOT NULL,
			  `password` VARCHAR( 255 ) NOT NULL,
			  `email` VARCHAR( 255 ) NOT NULL,
			  PRIMARY KEY (`id`),
			  UNIQUE KEY `username` (`username`),
			  KEY `password` (`password`)
			)
			");
			
			// Add account
			$db->query("INSERT INTO `users` (
				`type` ,
				`username` ,
				`password` ,
				`email`
				)
				VALUES ('admin', '" . $_POST['account_username'] ."', '" . md5($_POST['account_password']) ."', '" . $_POST['account_email'] ."');
			");
			
			// Create templates table
			$db->query("CREATE TABLE `templates` (
				`id` INT(11) unsigned NOT NULL auto_increment,
				`template_name` VARCHAR( 255 ) NOT NULL,
				`template_description` TEXT NOT NULL,
				`template_package` VARCHAR( 255 ) NOT NULL,
				PRIMARY KEY ( `id` )
			)");
			
			// Create apps table
			$db->query("CREATE TABLE `apps` (
			  `id` INT( 11 ) unsigned NOT NULL auto_increment,
			  `clientid` INT( 11 ) unsigned NOT NULL, 
			  `app_name` VARCHAR( 255 ) NOT NULL,
			  `app_description` MEDIUMTEXT NOT NULL,
			  `app_version` VARCHAR( 255 ) NOT NULL,
			  `app_versioncode` VARCHAR( 255 ) NOT NULL,
			  `app_identifier` VARCHAR( 255 ) NOT NULL,
			  `app_templateid` INT( 11 ) unsigned NOT NULL,
			  `app_debug` TINYINT( 1 ) NOT NULL,
			  `android_cert_title` VARCHAR( 255 ) NOT NULL,
			  `android_cert_alias` VARCHAR( 255 ) NOT NULL,
			  `android_cert_keypass` VARCHAR( 255 ) NOT NULL,
			  `android_cert_keystorepass` VARCHAR( 255 ) NOT NULL,
			  `ios_cert_title` VARCHAR( 255 ) NOT NULL,
			  `ios_cert_password` VARCHAR( 255 ) NOT NULL,
			  `blackberry_cert_title` VARCHAR( 255 ) NOT NULL,
			  `blackberry_cert_pass` VARCHAR( 255 ) NOT NULL,
			  `push_android_enabled` TINYINT( 1 ) NOT NULL,
			  `push_android_apikey` VARCHAR( 255 ) NOT NULL,
			  `push_ios_enabled` TINYINT( 1 ) NOT NULL,
			  `push_ios_keypass` VARCHAR( 255 ) NOT NULL,
			  `push_ios_devmode` TINYINT( 1 ) NOT NULL,
			  `stats_enabled` TINYINT( 1 ) NOT NULL,
			  `stats_accountid` VARCHAR( 255 ) NOT NULL,
			  PRIMARY KEY  (`id`)
			)
			");
			
			// Create settings table
			$db->query("CREATE TABLE `settings` (
				`setting` VARCHAR( 255 ) NOT NULL,
				`value` VARCHAR( 255 ) NOT NULL,
				PRIMARY KEY ( `setting` )
			)");
			
			// Add settings
			$db->query("INSERT INTO `settings` (
				`setting` ,
				`value`
				)
				VALUES
				('buildtool_purchaseid', ''),
				('phonegap_username', ''),
				('phonegap_password', ''),
				('phonegap_token', ''),
				('allow_registration', '0');
			");
			
			// Create push_devices table
			$db->query("CREATE TABLE `push_devices` (
			  `id` INT( 11 ) unsigned NOT NULL auto_increment,
			  `app_id` INT( 11 ) unsigned NOT NULL,
			  `device_id` mediumtext NOT NULL,
			  `os` varchar(255) NOT NULL,
			  PRIMARY KEY (`id`)
			)");
		}
		else
		{
			$this->errors = $db->errors;
		}
	}
	
	/* Write the config file. */
	private function writeConfig()
	{
		$config = __SYSTEM_DIR__ .'config.php';
		
		$configdata = '<?php
/* --------------------------------------------------------------------------------------------------- 
 * BuildTool - App Builder
 * 
 * @author	AppResponsive <http://robinbonnes.nl/>
 * @version	1.5
 *
 * Copyright (C) 2013 - 2014 AppResponsive. All rights reserved.
 * 
 * -------------------------------------------------------------------------------------------------- */

/* Check access */
if(!defined(\'access\') or !access) die(\'This file cannot be directly accessed.\');

$config = array();

/**
 * Database details (MySQL)
 */
$config[\'db_host\'] = \'' . $this->db_host . '\';
$config[\'db_name\'] = \'' . $this->db_database . '\';
$config[\'db_user\'] = \'' . $this->db_username . '\';
$config[\'db_pass\'] = \'' . $this->db_password . '\';

/**
 * BuildTool information
 */
$config[\'bt_version\'] = \'' . __BT_VERSION__ . '\';

?>';

		$result = file_put_contents($config, $configdata);

		if ($result === false)
		{
			$this->errors['Config'] = "<img class=\"requirement-icon\" src=\"./assets/images/icons/cross.png\" width=\"32\" height=\"32\"><div class=\"requirement\"><font color=\"#FF0000\">Can't write config settings.</font></div>";
		}
	}
}
?>