<?php

/* --------------------------------------------------------------------------------------------------- 
 * BuildTool - App Builder
 * 
 * @author	AppResponsive <http://robinbonnes.nl/>
 * @version	1.5
 *
 * Copyright (C) 2013 - 2014 AppResponsive. All rights reserved.
 * 
 * -------------------------------------------------------------------------------------------------- */

/* Check access */
if(!defined('access') or !access) die('This file cannot be directly accessed.');

class Login {

	/* array containing errors during login */
	private $errors = array();
	
	/* Database connection */
	private $db;
	
	/* Constructor */
	public function __construct()
	{
		/* Assign Database */
		$this->db = Database::getInstance();
		
		/* Check if login form has been posted. */
		$this->check();
		
		/* Show login form. */
		$this->showLogin();
	}
	
	/* Check if Form has been posted. */
	private function check()
	{
		// Login
		if(isset($_POST['login']))
		{
			if(isset($_POST['username']) && isset($_POST['password']))
			{
				// Check login
				$this->db->prepare("SELECT COUNT(*) FROM users WHERE username = ? AND password = ?");
				$this->db->bindParam(1, $_POST['username']);
				$this->db->bindParam(2, md5($_POST["password"]));
				$this->db->execute();
				$login = $this->db->fetch();
				
				if($login[0] == 0)
				{
					// No account with credentials
					$header = new Template("header.tpl.php");
					$header->set("title", "BuildTool - Login");
					$header->set("subtitle", "Please login to BuildTool.");
					$footer = new Template("footer.tpl.php");
					$body = new Template("login.tpl.php");
					$body->set("header", $header->parse());
					$body->set("footer", $footer->parse());
					if($this->registration_allowed())
					{
						$body->set("registration", true);
					}
					else
					{
						$body->set("registration", false);
					}
					$body->set("type", "login");
					$body->set("error", "<img class=\"message-icon\" src=\"./assets/images/icons/cross.png\">Couldn't login to BuildTool:<br>Wrong username/password.");
					$body->render();
				}
				else
				{
					// User
					$_SESSION["user"] = $_POST['username'];
					
					// Login Hash
					$_SESSION["hash"] = md5($_POST['username'] . md5($_POST["password"]) . $_SERVER['REMOTE_ADDR']);

					// Load dashboard (default/landing)
					require_once(__CONTROLLERS_DIR__ .'/dashboard.php');
					new Dashboard();
					die();
				}
			}
			else
			{
				// Show login
				$header = new Template("header.tpl.php");
				$header->set("title", "BuildTool - Login");
				$header->set("subtitle", "Please login to BuildTool.");
				$footer = new Template("footer.tpl.php");
				$body = new Template("login.tpl.php");
				$body->set("header", $header->parse());
				$body->set("footer", $footer->parse());
				if($this->registration_allowed())
				{
					$body->set("registration", true);
				}
				else
				{
					$body->set("registration", false);
				}
				$body->set("type", "login");
				$body->set("title", "BuildTool - Login");
				$body->render();
			}
			
			die();
		}
		
		// Forgot Password
		if(isset($_GET['forgot-password']))
		{				
			if(isset($_POST['email']))
			{
				$this->db->prepare("SELECT COUNT(*) FROM users WHERE email = ?");
				$this->db->bindParam(1, $_POST['email']);
				$this->db->execute();
				$login = $this->db->fetch();
				
				if($login[0] != 0)
				{
					/* Set up Template */
					$header = new Template("header.tpl.php");
					$header->set("title", "BuildTool - Forgot Password");
					$header->set("subtitle", "Reset your password.");
					$footer = new Template("footer.tpl.php");
					$body = new Template("login.tpl.php");
					$body->set("header", $header->parse());
					$body->set("footer", $footer->parse());
					if($this->registration_allowed())
					{
						$body->set("registration", true);
					}
					else
					{
						$body->set("registration", false);
					}
					$body->set("type", "forgot-password");
					
					/* Generate new pass */
					require_once(__SYSTEM_DIR__ .'/functions.php');
					$f = new Functions();
					$newpassword = $f->randomPassword();
										
					/* Send mail */
					require_once(__SYSTEM_DIR__ .'/classes/mailer.class.php');
					$m = new Mailer();
					$m->from = "info@" . $_SERVER['SERVER_NAME'];
					$m->to = $_POST['email'];
					$m->subject = "BuildTool - New Password";
					$m->body = "Hello\r\n\r\nNew BuildTool password requested by ip: " . $_SERVER['REMOTE_ADDR'] . ".\r\nPassword: " . $newpassword . "\r\n\r\nGoodluck!";
					if($m->send())
					{
						$body->set("error", "<img class=\"message-icon\" src=\"./assets/images/icons/tick.png\">New password has been sent to email:" . $_POST['email']);
						
						/* Update password in database */
						$this->db->prepare("UPDATE users SET password = ? WHERE email = ?");
						$this->db->bindParam(1, md5($newpassword));
						$this->db->bindParam(2, $_POST['email']);
						$this->db->execute();
					}
					else
					{
						$body->set("error", "<img class=\"message-icon\" src=\"./assets/images/icons/cross.png\">Could not send password to email: " . $_POST['email'] . " Password <b>not</b> changed: " . $m->error);
					}

					/* Render template */
					$body->render();
				}
				else
				{
					/* Error: No account for email */
					$header = new Template("header.tpl.php");
					$header->set("title", "BuildTool - Forgot Password");
					$header->set("subtitle", "Reset your password.");
					$footer = new Template("footer.tpl.php");
					$body = new Template("login.tpl.php");
					$body->set("header", $header->parse());
					$body->set("footer", $footer->parse());
					$body->set("type", "forgot-password");
					if($this->registration_allowed())
					{
						$body->set("registration", true);
					}
					else
					{
						$body->set("registration", false);
					}
					$body->set("error", "<img class=\"message-icon\" src=\"./assets/images/icons/cross.png\">No account found for email: <br>" . $_POST['email']);
					$body->render();
				}
			}
			else
			{
				// Show forgot password
				$header = new Template("header.tpl.php");
				$header->set("title", "BuildTool - Forgot Password");
				$header->set("subtitle", "Reset your password.");
				$footer = new Template("footer.tpl.php");
				$body = new Template("login.tpl.php");
				$body->set("header", $header->parse());
				$body->set("footer", $footer->parse());
				$body->set("type", "forgot-password");
				if($this->registration_allowed())
				{
					$body->set("registration", true);
				}
				else
				{
					$body->set("registration", false);
				}
				$body->set("error", "Please provide your email.");
				$body->render();
			}
			
			die();
		}
		
		if(isset($_GET['register']))
		{
			if($this->registration_allowed())
			{
				if(isset($_POST['register']))
				{
					if(!empty($_POST["username"]) && !empty($_POST["email"]) && !empty($_POST["password"]) && !empty($_POST["password_repeat"]))
					{
						if($_POST["password"] == $_POST["password_repeat"])
						{
							// Check if username or email already exists
							$this->db->prepare("SELECT * FROM users WHERE username = ? or email = ?");
							$this->db->bindParam(1, $_POST['username']);
							$this->db->bindParam(2, $_POST['email']);
							$this->db->execute();
							$exists = $this->db->fetch();
							
							if($exists == 0)
							{
								// Add to database
								$this->db->prepare("INSERT INTO users(type, username, password, email) VALUES (?, ?, ?, ?)");
								$this->db->bindParam(1, "client");
								$this->db->bindParam(2, $_POST['username']);
								$this->db->bindParam(3, md5($_POST['password']));
								$this->db->bindParam(4, $_POST['email']);
								$this->db->execute();
							
								// Send mail
								require_once(__SYSTEM_DIR__ .'/classes/mailer.class.php');
								$m = new Mailer();
								$m->from = "info@" . $_SERVER['SERVER_NAME'];
								$m->to = $_POST['email'];
								$m->subject = "BuildTool - Registration";
								$m->body = "Hello" . $_POST['username'] . ",\r\n\r\nYour BuildTool account is registered on: " . __URL__ . "\r\nThe login details are:\r\nUsername: " . $_POST['username'] . "Password: " . $_POST['password'] . "\r\n\r\nGoodluck building apps!";
								$m->send();
								
								// Output
								$header = new Template("header.tpl.php");
								$header->set("title", "BuildTool - Login");
								$header->set("subtitle", "Please login to BuildTool.");
								$footer = new Template("footer.tpl.php");
								$body = new Template("login.tpl.php");
								$body->set("header", $header->parse());
								$body->set("footer", $footer->parse());
								if($this->registration_allowed())
								{
									$body->set("registration", true);
								}
								else
								{
									$body->set("registration", false);
								}
								$body->set("type", "login");
								$body->set("error", "<img class=\"message-icon\" src=\"./assets/images/icons/tick.png\">Account registered. Please login below.");
							}
							else
							{
								$header = new Template("header.tpl.php");
								$header->set("title", "BuildTool - Register");
								$header->set("subtitle", "Please register to BuildTool.");
								$footer = new Template("footer.tpl.php");
								$body = new Template("login.tpl.php");
								$body->set("header", $header->parse());
								$body->set("footer", $footer->parse());
								if($this->registration_allowed())
								{
									$body->set("registration", true);
								}
								else
								{
									$body->set("registration", false);
								}
								$body->set("type", "register");
								$body->set("error", "<img class=\"message-icon\" src=\"./assets/images/icons/cross.png\">Username or email already registered.");
							}
						}
						else 
						{
							$header = new Template("header.tpl.php");
							$header->set("title", "BuildTool - Register");
							$header->set("subtitle", "Please register to BuildTool.");
							$footer = new Template("footer.tpl.php");
							$body = new Template("login.tpl.php");
							$body->set("header", $header->parse());
							$body->set("footer", $footer->parse());
							if($this->registration_allowed())
							{
								$body->set("registration", true);
							}
							else
							{
								$body->set("registration", false);
							}
							$body->set("type", "register");
							$body->set("error", "<img class=\"message-icon\" src=\"./assets/images/icons/cross.png\">Passwords doesn't match. Try again.");
						}
					}
					else
					{
						$header = new Template("header.tpl.php");
						$header->set("title", "BuildTool - Register");
						$header->set("subtitle", "Please register to BuildTool.");
						$footer = new Template("footer.tpl.php");
						$body = new Template("login.tpl.php");
						$body->set("header", $header->parse());
						$body->set("footer", $footer->parse());
						if($this->registration_allowed())
						{
							$body->set("registration", true);
						}
						else
						{
							$body->set("registration", false);
						}
						$body->set("type", "register");
						$body->set("error", "<img class=\"message-icon\" src=\"./assets/images/icons/cross.png\">Please fill in all fields.");
					}
					$body->render();
				}
				else
				{
					// Show register
					$this->showRegister();
				}
			}
			else
			{
				$this->showLogin();
			}
			
			die();
		}
	}

	/* Check if registration is allowed */
	private function registration_allowed()
	{
		$this->db->prepare("SELECT value FROM settings WHERE `setting` = 'allow_registration'");
		$this->db->execute();
		$result = $this->db->fetch();
		return $result[0];
	}
	
	/* Show Register Form. */
	private function showRegister()
	{
		$header = new Template("header.tpl.php");
		$header->set("title", "BuildTool - Register");
		$header->set("subtitle", "Please register to BuildTool.");
		$footer = new Template("footer.tpl.php");
		$body = new Template("login.tpl.php");
		$body->set("header", $header->parse());
		$body->set("footer", $footer->parse());
		if($this->registration_allowed())
		{
			$body->set("registration", true);
		}
		else
		{
			$body->set("registration", false);
		}
		$body->set("type", "register");
		$body->render();
	}
	
	/* Show Login Form. */
	private function showLogin()
	{
		$header = new Template("header.tpl.php");
		$header->set("title", "BuildTool - Login");
		$header->set("subtitle", "Please login to BuildTool.");
		$footer = new Template("footer.tpl.php");
		$body = new Template("login.tpl.php");
		$body->set("header", $header->parse());
		$body->set("footer", $footer->parse());
		if($this->registration_allowed())
		{
			$body->set("registration", true);
		}
		else
		{
			$body->set("registration", false);
		}
		$body->set("type", "login");
		$body->render();
	}

}
?>