<?php

/* --------------------------------------------------------------------------------------------------- 
 * BuildTool - App Builder
 * 
 * @author	AppResponsive <http://robinbonnes.nl/>
 * @version	1.5
 *
 * Copyright (C) 2013 - 2014 AppResponsive. All rights reserved.
 * 
 * -------------------------------------------------------------------------------------------------- */

/* Check access */
if(!defined('access') or !access) die('This file cannot be directly accessed.');

class Settings {

	/* Database connection */
	private $db;
	
	/* Users array */
	public $users;
	
	/* Phonegap Settings */
	public $phonegap_username;
	public $phonegap_password;
	public $phonegap_token;
	
	/* BuildTool Settings */
	public $buildtool_purchaseid;
	public $allow_registration;
	
	/* Constructor */
	public function __construct()
	{
		/* Assign Database */
		$this->db = Database::getInstance();
		
		/* Check if POST/GET is done. */
		$this->check();
		
		/* Get all settings */
		$this->getSettings();
		
		/* Get all users */
		$this->getUsers();
		
		/* Show Settings Overview. */
		$this->showSettingsOverview();
	}
	
	/* Check if some post is done. */
	private function check()
	{
		// Get user
		$this->db->prepare("SELECT * FROM users WHERE username = ?");
		$this->db->bindParam(1, $_SESSION['user']);
		$this->db->execute();
		$user = $this->db->fetch();
		$username = $user['username'];
		$user_email = $user['email'];
		$user_id = $user['id'];
		$user_type = $user['type'];
		
		// Logout
		if(isset($_GET['logout']))
		{
			// Destroy Session
			session_destroy();
			$host	=	"http://" . $_SERVER["HTTP_HOST"] . rtrim(dirname($_SERVER['PHP_SELF']), '/\\');
			header("Location: $host");
		}
		
		// Settings form has been submitted
		if(isset($_POST['buildtool_purchaseid']))
		{
			// Return if user is client
			if($user_type == "client")
			{
				// Not allowed
				$this->showSettingsOverview();
			}
			
			// Save to database
			$this->db->prepare("UPDATE settings SET value = ? WHERE setting = ?");
			$this->db->bindParam(1, $_POST['phonegap_username']);
			$this->db->bindParam(2, "phonegap_username");
			$this->db->execute();
			$this->db->prepare("UPDATE settings SET value = ? WHERE setting = ?");
			$this->db->bindParam(1, $_POST['phonegap_password']);
			$this->db->bindParam(2, "phonegap_password");
			$this->db->execute();
			$this->db->prepare("UPDATE settings SET value = ? WHERE setting = ?");
			$this->db->bindParam(1, $_POST['phonegap_token']);
			$this->db->bindParam(2, "phonegap_token");
			$this->db->execute();
			$this->db->prepare("UPDATE settings SET value = ? WHERE setting = ?");
			$this->db->bindParam(1, $_POST['buildtool_purchaseid']);
			$this->db->bindParam(2, "buildtool_purchaseid");
			$this->db->execute();
			
			$this->db->prepare("UPDATE settings SET value = ? WHERE setting = ?");
			if(isset($_POST['allow_registration']))
			{
				$this->db->bindParam(1, "1");
			}
			else
			{
				$this->db->bindParam(1, "0");
			}
			$this->db->bindParam(2, "allow_registration");
			$this->db->execute();
			
			// Output
			$header = new Template("header.tpl.php");
			$header->set("title", "BuildTool - Settings");
			$header->set("subtitle", "Make sure you configure Buildtool properly.");
			$footer = new Template("footer.tpl.php");
			$body = new Template("settings.tpl.php");
			$body->set("header", $header->parse());
			$body->set("footer", $footer->parse());
			$body->set("username", $_SESSION["user"]);
			$this->getSettings();
			$body->set("phonegap_username",$this->phonegap_username);
			$body->set("phonegap_password",$this->phonegap_password);
			$body->set("phonegap_token",$this->phonegap_token);
			$body->set("buildtool_purchaseid",$this->buildtool_purchaseid);
			$body->set("allow_registration",$this->allow_registration);
			$this->getUsers();
			$body->set("users", $this->users);
			$body->set("success", "Settings succesfully saved!");
			$body->render();
			
			die();
		}
		
		// Save Account Settings (admins and clients)
		if(isset($_POST["action"]) && $_POST["action"] == "edit")
		{
			if(isset($_POST["account_email"]) && !empty($_POST["account_email"]))
			{
				if($user_type == "client")
				{
					if(!empty($_POST["account_password"]) && !empty($_POST["account_passwordrepeat"]) && $_POST["account_password"] == $_POST["account_passwordrepeat"])
					{
						$this->db->prepare("UPDATE users SET type = ?, username = ?, password = ?, email = ? WHERE id = ?");
						
						$this->db->bindParam(1, "client");
						$this->db->bindParam(2, $username);
						$this->db->bindParam(3, md5($_POST['account_password']));
						$this->db->bindParam(4, $_POST['account_email']);
						$this->db->bindParam(5, $user_id);
						
						$this->db->execute();
					}
					else
					{
						$this->db->prepare("UPDATE users SET type = ?, username = ?, email = ? WHERE id = ?");

						$this->db->bindParam(1, "client");
						$this->db->bindParam(2, $username);
						$this->db->bindParam(3, $_POST['account_email']);
						$this->db->bindParam(4, $user_id);
						
						$this->db->execute();
					}
					
					// Output
					$header = new Template("header.tpl.php");
					$header->set("title", "BuildTool - Edit user");
					$header->set("subtitle", "Edit a user.");
					$footer = new Template("footer.tpl.php");
					$body = new Template("user_edit.tpl.php");
					$body->set("type", "client");
					$body->set("account_username", $user['username']);
					$body->set("account_email", $_POST['account_email']);
					$body->set("account_type", $user['type']);
					$body->set("success", "Account has been updated succesfully!");
				}
				else
				{
					if(isset($_POST['account_username']) && !empty($_POST['account_username']))
					{
						if(!empty($_POST["account_password"]))
						{
							$this->db->prepare("UPDATE users SET type = ?, username = ?, password = ?, email = ? WHERE id = ?");
							$this->db->bindParam(1, $_POST['account_type']);
							$this->db->bindParam(2, $_POST['account_username']);
							$this->db->bindParam(3, md5($_POST['account_password']));
							$this->db->bindParam(4, $_POST['account_email']);
							$this->db->bindParam(5, $_GET['id']);
							$this->db->execute();
						}
						else
						{
							$this->db->prepare("UPDATE users SET type = ?, username = ?, email = ? WHERE id = ?");
							$this->db->bindParam(1, $_POST['account_type']);
							$this->db->bindParam(2, $_POST['account_username']);
							$this->db->bindParam(3, $_POST['account_email']);
							$this->db->bindParam(4, $_GET['id']);
							$this->db->execute();
						}
						
						// Output
						$header = new Template("header.tpl.php");
						$header->set("title", "BuildTool - Settings");
						$header->set("subtitle", "Make sure you configure Buildtool properly.");
						$footer = new Template("footer.tpl.php");
						$body = new Template("settings.tpl.php");
						$this->getSettings();
						$body->set("phonegap_username",$this->phonegap_username);
						$body->set("phonegap_password",$this->phonegap_password);
						$body->set("phonegap_token",$this->phonegap_token);
						$body->set("buildtool_purchaseid",$this->buildtool_purchaseid);
						$body->set("allow_registration",$this->allow_registration);
						$this->getUsers();
						$body->set("users", $this->users);
						$body->set("success", "Account has been updated succesfully!");
					}
					else
					{
						$header = new Template("header.tpl.php");
						$header->set("title", "BuildTool - Edit user");
						$header->set("subtitle", "Edit a user.");
						$footer = new Template("footer.tpl.php");
						$body = new Template("user_edit.tpl.php");
						$body->set("type", $user_type);
						$body->set("account_username", $username);
						$body->set("account_email", $user_email);
						$body->set("account_type", $user_type);
						$body->set("error", "Please fill in an account username!");
					}
				}		
				$body->set("header", $header->parse());
				$body->set("footer", $footer->parse());
				$body->set("username", $_SESSION["user"]);
				$body->render();
				die();	
			}
			else
			{
				$this->showSettingsOverview();
			}
		}
		
		// User actions
		if(isset($_GET['action']))
		{
			// Edit user (admins and clients)
			if($_GET['action'] == "user-edit")
			{
				if(isset($_GET['id']) || isset($_GET['name']))
				{
					if(isset($_GET['id']))
					{
						if($user_type == "client")
						{
							if($user_id != $_GET['id'])
							{
								// Not allowed
								$this->showSettingsOverview();
							}
						}
			
						$this->db->prepare("SELECT * FROM users WHERE id = ?");
						$this->db->bindParam(1, $_GET['id']);
					}
					else
					{
						if($user_type == "client")
						{
							if($username != $_GET['name'])
							{
								// Not allowed
								$this->showSettingsOverview();
							}
						}
					
						$this->db->prepare("SELECT * FROM users WHERE username = ?");
						$this->db->bindParam(1, $_GET['name']);
					}
					
					$this->db->execute();
					$userentry = $this->db->fetch();
					
					if($userentry != 0)
					{
						$header = new Template("header.tpl.php");
						$header->set("title", "BuildTool - Edit user");
						$header->set("subtitle", "Edit a user.");
						$footer = new Template("footer.tpl.php");
						$body = new Template("user_edit.tpl.php");
						$body->set("type", $user_type);
						$body->set("header", $header->parse());
						$body->set("footer", $footer->parse());
						$body->set("username", $_SESSION["user"]);
						$body->set("account_username", $userentry['username']);
						$body->set("account_email", $userentry['email']);
						$body->set("account_type", $userentry['type']);
						$body->render();
					}
					else
					{
						$this->showSettingsOverview();
					}
				}
				else
				{
					$this->showSettingsOverview();
				}
			}
			
			// Delete user (admins only)
			else if($_GET['action'] == "user-delete")
			{
				// Return if user is client
				if($user_type == "client")
				{
					// Not allowed
					$this->showSettingsOverview();
				}
			
				// Output
				$header = new Template("header.tpl.php");
				$header->set("title", "BuildTool - Settings");
				$header->set("subtitle", "Make sure you configure Buildtool properly.");
				$footer = new Template("footer.tpl.php");
				$body = new Template("settings.tpl.php");
				$body->set("header", $header->parse());
				$body->set("footer", $footer->parse());
				$body->set("username", $_SESSION["user"]);
		
				// Check if isn't the last account left in the database
				$this->db->prepare("SELECT * FROM users");
				$this->db->execute();
				$users = $this->db->fetchAll();
				
				if(count($users) > 1)
				{
					// Delete ID
					if(isset($_GET['id']))
					{
						$this->db->prepare("SELECT * FROM users WHERE id = ?");
						$this->db->bindParam(1, $_GET['id']);
						$this->db->execute();
						$userentry = $this->db->fetch();
							
						if($userentry != 0)
						{
							// Delete entry from database
							$this->db->prepare("DELETE FROM users WHERE id = ?");
							$this->db->bindParam(1, $_GET['id']);
							$this->db->execute();
							$body->set("success", "Account succesfully deleted!");
						}
						else
						{
							$body->set("error", "ID doesn't exist!");
						}
					}
					else
					{
						$body->set("error", "Can't delete account without ID!");
					}
				}
				else
				{
					// Last account in database, can't delete
					$body->set("error", "Can't delete last account in database");
				}
				
				$this->getSettings();
				$body->set("phonegap_username",$this->phonegap_username);
				$body->set("phonegap_password",$this->phonegap_password);
				$body->set("phonegap_token",$this->phonegap_token);
				$body->set("buildtool_purchaseid",$this->buildtool_purchaseid);
				$body->set("allow_registration",$this->allow_registration);
				$this->getUsers();
				$body->set("users", $this->users);
				$body->render();
			}
			
			// Add user (admins only)
			else
			{
				// Return if user is client
				if($user_type == "client")
				{
					// Not allowed
					$this->showSettingsOverview();
				}
			
				if(isset($_POST["account_username"]))
				{
					$header = new Template("header.tpl.php");
					$header->set("title", "BuildTool - Add user");
					$header->set("subtitle", "Add a new user.");
					$footer = new Template("footer.tpl.php");
					
					if($_POST["account_username"] != "")
					{
						if(isset($_POST["account_password"]) && $_POST["account_password"] != "")
						{
							if(isset($_POST["account_email"]) && $_POST["account_email"] != "")
							{
								// Check if username already exists
								$this->db->prepare("SELECT * FROM users WHERE username = ?");
								$this->db->bindParam(1, $_POST['account_username']);
								$this->db->execute();
								$exists = $this->db->fetch();
								
								if($exists != 0)
								{
									$body = new Template("user_new.tpl.php");
									$body->set("error", "Username already exists!");
								}
								else
								{
									// Add to database
									$this->db->prepare("INSERT INTO users(type, username, password, email) VALUES (?, ?, ?, ?)");
									$this->db->bindParam(1, $_POST['account_type']);
									$this->db->bindParam(2, $_POST['account_username']);
									$this->db->bindParam(3, md5($_POST['account_password']));
									$this->db->bindParam(4, $_POST['account_email']);
									$this->db->execute();
								
									// Output
									$body = new Template("settings.tpl.php");
									$this->getSettings();
									$body->set("phonegap_username",$this->phonegap_username);
									$body->set("phonegap_password",$this->phonegap_password);
									$body->set("phonegap_token",$this->phonegap_token);
									$body->set("buildtool_purchaseid",$this->buildtool_purchaseid);
									$body->set("allow_registration",$this->allow_registration);
									$this->getUsers();
									$body->set("users", $this->users);
									$body->set("success", "Account has been created!");
								}
							}
							else
							{
								$body = new Template("user_new.tpl.php");
								$body->set("error", "Please fill in an account email!");
							}
						}
						else 
						{
							$body = new Template("user_new.tpl.php");
							$body->set("error", "Please fill in an account password!");
						}
					}
					else
					{
						$body = new Template("user_new.tpl.php");
						$body->set("error", "Please fill in an account username!");
					}
					
					$body->set("header", $header->parse());
					$body->set("footer", $footer->parse());
					$body->set("username", $_SESSION["user"]);
					$body->render();
				}
				else
				{
					// Output
					$header = new Template("header.tpl.php");
					$header->set("title", "BuildTool - Add user");
					$header->set("subtitle", "Add a new user.");
					$footer = new Template("footer.tpl.php");
					$body = new Template("user_new.tpl.php");
					$body->set("header", $header->parse());
					$body->set("footer", $footer->parse());
					$body->set("username", $_SESSION["user"]);
					$body->render();
				}			
			}
			die();
		}
		
		// Load page
		if(isset($_GET['page']))
		{
			if(file_exists(__CONTROLLERS_DIR__ .'/' . $_GET['page'] . '.php'))
			{
				require_once(__CONTROLLERS_DIR__ .'/' . $_GET['page'] . '.php');
				$classname = ucfirst(strtolower($_GET['page']));
				if(get_class($this) != $classname)
				{
					if(isset($_GET[1]))
					{
						new $classname($_GET[1]);
					}
					else
					{
						new $classname();
					}
					die();
				}
			}
		}
		
	}
	
	/* Get all settings */
	private function getSettings()
	{
		$this->db->prepare("SELECT * FROM settings");
		$this->db->execute();
		$settings = $this->db->fetchAll();

		foreach($settings as $setting )
		{
			$this->$setting['setting'] = $setting['value'];
			
		}
	}
	
	/* Get all users */
	public function getUsers()
	{
		$this->db->prepare("SELECT * FROM users");
		$this->db->execute();
		$users = $this->db->fetchAll();

		foreach ($users as $user)
		{
			$userinfo['user_type']			=	$user['type'];
			$userinfo['user_name']			=	$user['username'];
			$userinfo['user_email']			=	$user['email'];
			$userinfo['user_actions']		=	"<a class='button button-rounded button-primary' href='?page=settings&amp;action=user-edit&amp;id=" . $user['id'] . "'><i class='icon-cog'></i></a>
												<a class='button button-rounded button-caution' href='?page=settings&amp;action=user-delete&amp;id=" . $user['id'] . "'><i class='icon-trash'></i></a>";
			$this->users[]	= $userinfo;
		}
	}
	
	/* Show Settings Overview. */
	private function showSettingsOverview()
	{
		// Get user
		$this->db->prepare("SELECT * FROM users WHERE username = ?");
		$this->db->bindParam(1, $_SESSION['user']);
		$this->db->execute();
		$user = $this->db->fetch();
		$username = $user['username'];
		$user_id = $user['id'];
		$user_type = $user['type'];

		// Client Settings
		if($user_type == "client")
		{
			$header = new Template("header.tpl.php");
			$header->set("title", "BuildTool - Edit user");
			$header->set("subtitle", "Edit a user.");
			$footer = new Template("footer.tpl.php");
			$body = new Template("user_edit.tpl.php");
			$body->set("type", "client");
			$body->set("header", $header->parse());
			$body->set("footer", $footer->parse());
			$body->set("username", $_SESSION["user"]);
			$body->set("account_username", $user['username']);
			$body->set("account_email", $user['email']);
			$body->set("account_type", $user['type']);
			$body->render();
		}
		
		// Admin Settings
		else
		{
			$header = new Template("header.tpl.php");
			$header->set("title", "BuildTool - Settings");
			$header->set("subtitle", "Make sure you configure Buildtool properly.");
			$footer = new Template("footer.tpl.php");
			$body = new Template("settings.tpl.php");
			$body->set("header", $header->parse());
			$body->set("footer", $footer->parse());
			$body->set("username", $_SESSION["user"]);
			$body->set("phonegap_username",$this->phonegap_username);
			$body->set("phonegap_password",$this->phonegap_password);
			$body->set("phonegap_token",$this->phonegap_token);
			$body->set("buildtool_purchaseid",$this->buildtool_purchaseid);
			$body->set("allow_registration",$this->allow_registration);
			$body->set("users", $this->users);
			$body->render();
		}
		die();
	}
	
}
?>